@@ -27,10 +27,11 @@
 
 #include <config.h>
 #include "gnome-vfs-cancellable-ops.h"
+#include "gnome-vfs-method.h"
 
+#include <glib/gmessages.h>
+#include <glib/gutils.h>
 #include <string.h>
-#include "gnome-vfs.h"
-#include "gnome-vfs-private.h"
 
 GnomeVFSResult
 gnome_vfs_open_uri_cancellable (GnomeVFSHandle **handle,
