@@ -1574,6 +1574,8 @@ move_items (const GList *source_uri_list,
 		int conflict_count;
 		int progress_result;
 
+		progress->progress_info->file_index++;
+
 		source_uri = (GnomeVFSURI *)source_item->data;
 		target_dir_uri = gnome_vfs_uri_get_parent ((GnomeVFSURI *)target_item->data);
 
@@ -1678,6 +1680,8 @@ link_items (const GList *source_uri_list,
 	/* go through the list of names, create a link to each item */
 	for (source_item = source_uri_list, target_item = target_uri_list; source_item != NULL;) {
 
+		progress->progress_info->file_index++;
+
 		source_uri = (GnomeVFSURI *)source_item->data;
 		source_reference = gnome_vfs_uri_to_string (source_uri, GNOME_VFS_URI_HIDE_NONE);
 
@@ -1860,9 +1864,8 @@ gnome_vfs_xfer_delete_items (const GList *source_uri_list,
 	progress->progress_info->bytes_total = 0;
 	call_progress (progress, GNOME_VFS_XFER_PHASE_COLLECTING);
 
-
 	result = count_items_and_size (source_uri_list,
-		GNOME_VFS_XFER_REMOVESOURCE | GNOME_VFS_XFER_RECURSIVE, progress, TRUE, FALSE);
+		GNOME_VFS_XFER_REMOVESOURCE | GNOME_VFS_XFER_RECURSIVE, progress, FALSE, FALSE);
 
 	/* When deleting, ignore the real file sizes, just count the same DEFAULT_SIZE_OVERHEAD
 	 * for each file.
@@ -1972,6 +1975,11 @@ gnome_vfs_destination_is_writable (const GnomeVFSURI *uri)
 		result = GNOME_VFS_OK;
 	}
 	
+	/* some methods only allow certain filenames (e.g. .desktop files) */
+	if (result == GNOME_VFS_ERROR_INVALID_URI) {
+		result = GNOME_VFS_OK;
+	}
+
 	gnome_vfs_uri_unref (test_uri);
 	return result;
 }
